@@ -47,6 +47,8 @@
 #define SEPARATOR_NAME         _("Space")
 #define ICON_SIZE              24
 #define UPDATE_MAX_SIZE_DELAY  100
+#define AUTO_SCROLL_STEP 4
+#define AUTO_SCROLL_THRESHOLD 20
 
 
 enum {
@@ -99,6 +101,8 @@ typedef struct {
 	GtkWidget     *p_minsize_spinbutton;
 	GtkWidget     *p_maxsize_spinbutton;
 	GtkWidget     *p_icon_size_checkbutton;
+	GtkWidget     *p_apps_scrolledwindow;
+	GtkAdjustment *p_apps_vadjustment;
 
 	int            n;
 	GtkTreeModel  *model;
@@ -186,6 +190,20 @@ help_cb (GtkWidget  *widget,
 }
 
 
+static void
+do_select_iter (DialogData  *data,
+	     GtkTreeIter *iter)
+{
+	GtkTreeSelection *selection;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
+	if (selection == NULL)
+                return;
+
+	gtk_tree_selection_select_iter (selection, iter);
+}
+
+
 static void
 select_and_view_iter (DialogData  *data,
 		      GtkTreeIter *iter)
@@ -283,7 +301,7 @@ move_up_cb (GtkWidget  *widget,
 }
 
 
-static void
+static gboolean
 move_down_cb (GtkWidget  *widget, 
 	      DialogData *data)
 {
@@ -294,16 +312,18 @@ move_down_cb (GtkWidget  *widget,
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
 	if (selection == NULL)
-                return;
+                return FALSE;
 	if (! gtk_tree_selection_get_selected (selection, NULL, &iter))
-		return;
+		return FALSE;
 
 	position = get_iter_position (data, &iter) + 1;
 	button = tree_view_move_iter_to (data, &iter, position, FALSE);
 	select_and_view_iter (data, &iter);
 	quick_box_reorder_child (data->quick_box, button, position);
-
+	
 	quick_lounge_save_order (data->quick_lounge);
+	
+	return FALSE;
 }
 
 
@@ -359,8 +379,7 @@ static void
 update_sensitivity (DialogData *data)
 {
 	GtkTreeSelection *selection;
-	GtkTreeIter       iter, first_iter, last_iter;
-	gboolean          first_selected, last_selected;
+	GtkTreeIter       iter;
 	GtkWidget        *button;
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (data->tree_view));
@@ -369,10 +388,6 @@ update_sensitivity (DialogData *data)
 	
 	if (! gtk_tree_selection_get_selected (selection, NULL, &iter)
 	    || (data->n == 0)) {
-		gtk_widget_set_sensitive (data->btn_up, FALSE);
-		gtk_widget_set_sensitive (data->btn_down, FALSE);
-		gtk_widget_set_sensitive (data->btn_top, FALSE);
-		gtk_widget_set_sensitive (data->btn_bottom, FALSE);
 		gtk_widget_set_sensitive (data->btn_edit, FALSE);
 		gtk_widget_set_sensitive (data->btn_delete, FALSE);
 
@@ -382,17 +397,6 @@ update_sensitivity (DialogData *data)
 	gtk_widget_set_sensitive (data->btn_edit, TRUE);
 	gtk_widget_set_sensitive (data->btn_delete, TRUE);
 	
-	gtk_tree_model_get_iter_first (data->model, &first_iter);
-	gtk_tree_model_iter_nth_child (data->model, &last_iter, NULL, data->n - 1);
-
-	first_selected = gtk_tree_selection_iter_is_selected (selection, &first_iter);
-	last_selected = gtk_tree_selection_iter_is_selected (selection, &last_iter);
-
-	gtk_widget_set_sensitive (data->btn_up, ! first_selected);
-	gtk_widget_set_sensitive (data->btn_top, ! first_selected);
-	gtk_widget_set_sensitive (data->btn_down, ! last_selected);
-	gtk_widget_set_sensitive (data->btn_bottom, ! last_selected);
-
 	gtk_tree_model_get (data->model,
 			    &iter,
 			    COLUMN_DATA, &button,
@@ -414,30 +418,32 @@ selection_changed_cb (GtkTreeSelection *selection,
 static void
 update_list (DialogData *data)
 {
-	GList        *scan;
+	GList        *children, *scan;
 	GtkBox       *box = GTK_BOX (data->quick_box);
 	GtkTreeIter   select_iter;
 
 	gtk_list_store_clear (GTK_LIST_STORE (data->model));
 
-	data->n = g_list_length (box->children);
-	for (scan = g_list_last (box->children); scan; scan = scan->prev) {
-		GtkBoxChild *child_info = scan->data;
+	children = gtk_container_get_children (GTK_CONTAINER (data->quick_box));
+	data->n = g_list_length (children);
+	for (scan = g_list_last (children); scan; scan = scan->prev) {
+		GtkWidget   *child_widget = scan->data;
 		QuickButton *button;
 		GdkPixbuf   *pixbuf;
 		GtkTreeIter  iter;
 		double       scale;
 		int          w, h;
 
-		if (! QUICK_IS_BUTTON (child_info->widget))
+		if (! QUICK_IS_BUTTON (child_widget))
 			continue;
-		button = QUICK_BUTTON (child_info->widget);
+
+		button = QUICK_BUTTON (child_widget);
 
 		if (QUICK_IS_SEPARATOR (button)) { /* Space */
 			gtk_list_store_prepend (GTK_LIST_STORE (data->model), 
 						&iter);
 
-			if (child_info->widget == data->select_button) 
+			if (child_widget == data->select_button) 
 				select_iter = iter;
 
 			gtk_list_store_set (GTK_LIST_STORE (data->model), 
@@ -465,7 +471,7 @@ update_list (DialogData *data)
 
 		gtk_list_store_prepend (GTK_LIST_STORE (data->model), &iter);
 
-		if (child_info->widget == data->select_button) 
+		if (child_widget == data->select_button) 
 			select_iter = iter;
 
 		gtk_list_store_set (GTK_LIST_STORE (data->model), &iter,
@@ -475,9 +481,10 @@ update_list (DialogData *data)
 				    -1);
 		g_object_unref (pixbuf);
 	}
+	g_list_free (children);
 
 	if (data->select_button != NULL) {
-		select_and_view_iter (data, &select_iter);
+		do_select_iter (data, &select_iter);
 		data->select_button = NULL;
 	}
 }
@@ -702,21 +709,26 @@ static GtkWidget *
 get_button_from_uri (QuickBox   *quick_box,
 		     const char *uri1)
 {
-	GtkBox *box = GTK_BOX (quick_box);
-	GList  *scan;
+	GtkBox    *box = GTK_BOX (quick_box);
+	GList     *children, *scan;
+	GtkWidget *retval = NULL;
 
-	for (scan = box->children; scan; scan = scan->next) {
-		GtkBoxChild *child = scan->data;
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget   *child_widget = scan->data;
 		const char  *uri2;
 
-		uri2 = g_object_get_data (G_OBJECT (child->widget), "uri");
+		uri2 = g_object_get_data (G_OBJECT (child_widget), "uri");
 		if (uri2 == NULL)
 			continue;
-		if (strcmp (uri1, uri2) == 0)
-			return child->widget;
+		if (strcmp (uri1, uri2) == 0) {
+			retval = child_widget;
+			break;
+		}
 	}
-
-	return NULL;
+	g_list_free (children);
+	
+	return retval;
 }
 
 
@@ -724,17 +736,23 @@ static int
 get_child_position (QuickBox  *quick_box,
 		    GtkWidget *widget)
 {
-	GList  *scan;
-	int     pos = 0;
-
-	for (scan = GTK_BOX (quick_box)->children; scan; scan = scan->next) {
-		GtkBoxChild *child = scan->data;
-
-		if (child->widget == widget)
-			return pos;
-		pos++;
+	GList    *children, *scan;
+	gboolean  found = FALSE;
+	int       pos;
+	
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (pos = 0, scan = children; scan; pos++, scan = scan->next) {
+		GtkWidget *child_widget = scan->data;
+		if (child_widget == widget) {
+			found = TRUE;
+			break;
+		}
 	}
-	return -1;
+
+	if (found)
+		return pos;
+	else
+		return -1;
 }
 
 
@@ -755,7 +773,7 @@ drag_data_received  (GtkWidget          *widget,
 	int           pos;
 	GtkTreePath  *pos_path;
 	GtkTreeViewDropPosition drop_pos;
-	char         *selection_data = data->data;
+	char         *selection_data = (char*)data->data;
 	GtkTreeIter   iter;
 
 
@@ -777,7 +795,7 @@ drag_data_received  (GtkWidget          *widget,
 
         /* separator 'uri' starts with a : as coded in drag_data_get() */
 	if (selection_data[0] == ':') { 
-		int          from_pos = atoi(selection_data + 1);
+		int          from_pos = atoi (selection_data + 1);
 		GtkWidget   *button;
 
 		gtk_tree_model_iter_nth_child (ddata->model, &iter, NULL, from_pos);
@@ -801,8 +819,9 @@ drag_data_received  (GtkWidget          *widget,
 		list = get_file_list_from_url_list (selection_data);
 		for (scan = list; scan; scan = scan->next) {
 			char      *src_uri = scan->data;
-			char      *uri;
+			char      *uri = NULL;
 			GtkWidget *button;
+			int        button_pos;
 
 			if (! is_desktop_file (src_uri))
 				continue;
@@ -810,39 +829,57 @@ drag_data_received  (GtkWidget          *widget,
 			uri = g_build_filename (quick_lounge->location,
 						file_name_from_path (src_uri),
 						NULL);
-			
-			if (strcmp (src_uri, uri) == 0) {
-				button = get_button_from_uri (quick_box, uri);
-				
-				if (button != NULL) {
-					int button_pos;
-					button_pos = get_child_position (quick_box, button);
-					if (button_pos != -1) 
-						quick_box_reorder_child (quick_box, button, pos);
-				}
-				
+
+			/* New laucher: add it to the list. */
+
+			if (strcmp (src_uri, uri) != 0) {
+				g_free (uri);
+				uri = quick_lounge_util__get_unique_uri (quick_lounge);
+				if (file_copy (src_uri, uri)) 
+					quick_box_add_button (quick_box, 
+							      uri, 
+							      pos);
 				g_free (uri);
 				continue;
 			}
 			
+			/* The launcher is already present, this means that
+			 * the laucher has been dragged from the properties 
+			 * dialog: change launcher position. */
+
+			button = get_button_from_uri (quick_box, uri);
 			g_free (uri);
-			uri = quick_lounge_util__get_unique_uri (quick_lounge);
-			
-			if (! file_copy (src_uri, uri)) {
-				g_free (uri);
+				
+			if (button == NULL) 
 				continue;
+				
+			button_pos = get_child_position (quick_box, button);
+			if (button_pos == -1) 
+				continue;
+			
+			if ((drop_pos == GTK_TREE_VIEW_DROP_BEFORE) ||
+			    (drop_pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)) {
+				if (button_pos < pos)
+					pos--;
 			}
 			
-			quick_box_add_button (quick_box, uri, ++pos);
+			if ((drop_pos == GTK_TREE_VIEW_DROP_AFTER) ||
+			    (drop_pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)) {
+				if (button_pos > pos)
+					pos++;
+			}
+			
+			quick_box_reorder_child (quick_box, button, pos);
 		}
+		
 		path_list_free (list);
 	}
-
+	
 	quick_lounge_save_order (quick_lounge);
 	dlg_properties_update (quick_lounge->prop_dialog);
 
 	gtk_tree_model_iter_nth_child (ddata->model, &iter, NULL, pos);
-	select_and_view_iter (ddata, &iter);
+	do_select_iter (ddata, &iter);
 }
 
 
@@ -857,7 +894,26 @@ drag_motion (GtkWidget          *widget,
 	DialogData              *ddata = extra_data;
 	GtkTreePath             *pos_path;
 	GtkTreeViewDropPosition  drop_pos;
-	gboolean                 result = FALSE;
+	GtkAdjustment           *adj = ddata->p_apps_vadjustment;
+
+#ifdef DEBUG
+	g_print ("(%d, %d) [%d, %d]\n", 
+		 x, y, 
+		 widget->allocation.width, widget->allocation.height);
+
+	g_print ("(%2.2f) [%2.2f, %2.2f]\n", 
+		 ddata->p_apps_vadjustment->value, 
+		 ddata->p_apps_vadjustment->lower, 
+		 ddata->p_apps_vadjustment->upper);
+#endif
+
+	if ((widget->allocation.height - y < AUTO_SCROLL_THRESHOLD)
+	    && (adj->value + widget->allocation.height < adj->upper)) {
+		gtk_adjustment_set_value (adj, gtk_adjustment_get_value (adj) + AUTO_SCROLL_STEP);
+
+	} else 	if ((y < AUTO_SCROLL_THRESHOLD) && (adj->value > 0.0)) {
+		gtk_adjustment_set_value (adj, gtk_adjustment_get_value (adj) - AUTO_SCROLL_STEP);
+	}
 
 	if (! gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (ddata->tree_view),
 						 x, y,
@@ -865,17 +921,19 @@ drag_motion (GtkWidget          *widget,
 						 &drop_pos)) 
 		pos_path = gtk_tree_path_new_first ();
 
-	if ((drop_pos != GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)
-	    && (drop_pos != GTK_TREE_VIEW_DROP_INTO_OR_AFTER)) {
-		gtk_tree_view_set_drag_dest_row  (GTK_TREE_VIEW (ddata->tree_view),
-						  pos_path,
-						  drop_pos);
-		result = TRUE;
-	}
+	if (drop_pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)
+		drop_pos = GTK_TREE_VIEW_DROP_BEFORE;
+	
+	else if (drop_pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)
+		drop_pos = GTK_TREE_VIEW_DROP_AFTER;
+	
+	gtk_tree_view_set_drag_dest_row  (GTK_TREE_VIEW (ddata->tree_view),
+					  pos_path,
+					  drop_pos);
 
 	gtk_tree_path_free (pos_path);
 
-	return result;
+	return TRUE;
 }
 
 
@@ -964,13 +1022,13 @@ drag_data_get  (GtkWidget        *widget,
 		gtk_selection_data_set (selection_data, 
 					selection_data->target,
 					8, 
-					pos_data,
-					strlen(pos_data));
+					(guchar*) pos_data,
+					strlen (pos_data));
 	} else 
 		gtk_selection_data_set (selection_data, 
 					selection_data->target,
 					8, 
-					uri,
+					(guchar*) uri,
 					strlen (uri));
 }
 
@@ -1019,6 +1077,9 @@ dlg_properties (QuickLounge  *quick_lounge,
 	data->p_minsize_spinbutton = glade_xml_get_widget (data->gui, "p_minsize_spinbutton");
 	data->p_maxsize_spinbutton = glade_xml_get_widget (data->gui, "p_maxsize_spinbutton");
 	data->p_icon_size_checkbutton = glade_xml_get_widget (data->gui, "p_icon_size_checkbutton");
+	data->p_apps_scrolledwindow = glade_xml_get_widget (data->gui, "p_apps_scrolledwindow");
+
+	data->p_apps_vadjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (data->p_apps_scrolledwindow));
 
 	btn_close = glade_xml_get_widget (data->gui, "p_close_button");
 	btn_help = glade_xml_get_widget (data->gui, "p_help_button");
@@ -1034,6 +1095,11 @@ dlg_properties (QuickLounge  *quick_lounge,
 	g_object_unref (data->model);
 	add_columns (GTK_TREE_VIEW (data->tree_view));
 
+	gtk_widget_set_sensitive (data->btn_up, TRUE);
+	gtk_widget_set_sensitive (data->btn_down, TRUE);
+	gtk_widget_set_sensitive (data->btn_top, TRUE);
+	gtk_widget_set_sensitive (data->btn_bottom, TRUE);
+
 	/**/
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->p_minsize_spinbutton), quick_box_get_min_visible_cols (data->quick_box));
