@@ -269,7 +269,7 @@ get_missing (GnomeIconTheme *theme,
 	char *filename = NULL;
 
 	filename = gnome_icon_theme_lookup_icon (theme,
-						 "gnome-unknown.png",
+						 "gnome-unknown",
 						 preffered_size,
 						 NULL,
 						 NULL);
@@ -654,7 +654,7 @@ drag_data_get  (GtkWidget        *widget,
         gtk_selection_data_set (selection_data, 
                                 selection_data->target,
                                 8, 
-				uri,
+				(guchar*)uri,
                                 strlen (uri));
 }
 
