@@ -150,7 +150,7 @@ about_verb_cb (BonoboUIComponent *uic,
 	gtk_show_about_dialog (NULL,
 			       "name", _("Launchers List"),
 			       "version", VERSION,
-			       "copyright", "Copyright \xc2\xa9 2002-2004 Free Software Foundation, Inc.",
+			       "copyright", "Copyright \xc2\xa9 2002-2005 Free Software Foundation, Inc.",
 			       "comments", _("Organize your favorite applications on the Panel"),
 			       "authors", authors,
 			       "documenters", documenters,
@@ -619,17 +619,25 @@ static int
 get_child_position (QuickBox  *quick_box,
 		    GtkWidget *widget)
 {
-	GList  *scan;
-	int     pos = 0;
+	GList    *children, *scan;
+	gboolean  found = FALSE;
+	int       pos;
 
-	for (scan = GTK_BOX (quick_box)->children; scan; scan = scan->next) {
-		GtkBoxChild *child = scan->data;
-
-		if (child->widget == widget)
-			return pos;
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (pos = 0, scan = children; scan; pos++, scan = scan->next) {
+		GtkWidget *child_widget = scan->data;
+		if (child_widget == widget) {
+			found = TRUE;
+			break;
+		}
 		pos++;
 	}
-	return -1;
+	g_list_free (children);
+
+	if (!found)
+		return -1;
+	else
+		return pos;
 }
 
 
@@ -821,8 +829,24 @@ add_space_cb (GtkWidget *button)
 
 
 static void
-activate_item_cb (GtkWidget *item,
-		  GtkWidget *button)
+execute_item_activate_cb (GtkWidget *item,
+			  GtkWidget *button)
+{
+	QuickBox    *quick_box;
+	QuickLounge *quick_lounge;
+
+	quick_box = (QuickBox*) button->parent;
+	quick_lounge = g_object_get_data (G_OBJECT (quick_box), "quick_lounge");
+	if (quick_lounge == NULL)
+		return;
+
+	gtk_button_clicked (GTK_BUTTON (button));
+}
+
+
+static void
+properties_activate_cb (GtkWidget *item,
+			GtkWidget *button)
 {
 	QuickBox    *quick_box;
 	QuickLounge *quick_lounge;
@@ -843,15 +867,21 @@ populate_context_menu (GtkWidget *popup_menu,
 {
 	GtkWidget *item;
 
-	if (! QUICK_IS_SEPARATOR (button)) {
-		item = gtk_image_menu_item_new_with_mnemonic (_("_Properties"));
-		gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), gtk_image_new_from_stock (GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU));
+	if (! QUICK_IS_SEPARATOR (button)) {		
+		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_EXECUTE, NULL);
 		gtk_widget_show_all (item);
-	
+		gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
 		g_signal_connect (G_OBJECT (item), 
 				  "activate",
-				  G_CALLBACK (activate_item_cb), 
+				  G_CALLBACK (execute_item_activate_cb), 
+				  button);
+
+		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
+		gtk_widget_show_all (item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
+		g_signal_connect (G_OBJECT (item), 
+				  "activate",
+				  G_CALLBACK (properties_activate_cb), 
 				  button);
 
 		/**/
@@ -905,11 +935,9 @@ populate_context_menu (GtkWidget *popup_menu,
 
 	/**/
 
-	item = gtk_image_menu_item_new_with_mnemonic (_("_Remove"));
-	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU));
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, NULL);
 	gtk_widget_show_all (item);
-
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
 	g_signal_connect_swapped (G_OBJECT (item), 
 				  "activate",
 				  G_CALLBACK (item_remove_cb), 
@@ -937,18 +965,20 @@ get_button_from_uri (QuickBox   *quick_box,
 		     const char *uri1)
 {
 	GtkBox *box = GTK_BOX (quick_box);
-	GList  *scan;
+	GList  *children, *scan;
 
-	for (scan = box->children; scan; scan = scan->next) {
-		GtkBoxChild *child = scan->data;
-		const char  *uri2;
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget  *child_widget = scan->data;
+		const char *uri2;
 
-		uri2 = g_object_get_data (G_OBJECT (child->widget), "uri");
+		uri2 = g_object_get_data (G_OBJECT (child_widget), "uri");
 		if (uri2 == NULL)
 			continue;
 		if (strcmp (uri1, uri2) == 0)
-			return child->widget;
+			return child_widget;
 	}
+	g_list_free (children);
 
 	return NULL;
 }
@@ -1027,30 +1057,31 @@ drag_data_received  (GtkWidget          *widget,
 void
 quick_lounge_save_order (QuickLounge *quick_lounge)
 {
-	GtkBox       *box = GTK_BOX (quick_lounge->quick_box);
 	PanelApplet  *applet = PANEL_APPLET (quick_lounge->applet);
 	GConfClient  *client;
 	char         *full_key;
-	GList        *scan;
+	GList        *children, *scan;
 	GSList       *uri_list = NULL;
-
-	for (scan = box->children; scan; scan = scan->next) {
-		GtkBoxChild      *child_info = scan->data;
-		GtkWidget        *button = child_info->widget;
+	
+	children = gtk_container_get_children (GTK_CONTAINER (quick_lounge->quick_box));
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget        *child_widget = scan->data;
 		GnomeDesktopItem *ditem;
 		
-		if (QUICK_IS_SEPARATOR (button)) {
+		if (QUICK_IS_SEPARATOR (child_widget)) {
 			uri_list = g_slist_prepend (uri_list, g_strdup (""));
 			continue;
 		}
 
-		ditem = g_object_get_data (G_OBJECT (button), "ditem");
+		ditem = g_object_get_data (G_OBJECT (child_widget), "ditem");
 		if (ditem != NULL) {
 			const char *uri;
 			uri = file_name_from_path (gnome_desktop_item_get_location (ditem));
 			uri_list = g_slist_prepend (uri_list, g_strdup (uri));
 		}
 	}
+	g_list_free (children);
+
 	uri_list = g_slist_reverse (uri_list);
 
 	full_key = panel_applet_gconf_get_full_key (applet, PREFS_ORDER);
@@ -1192,7 +1223,7 @@ quick_lounge_load_uri_async (QuickLounge  *quick_lounge,
 			     const char   *uri)
 {
 	QuickBox     *quick_box = quick_lounge->quick_box;
-	GList        *children;
+	GList        *children, *scan;
 	GtkContainer *container;
 
 	if (quick_lounge->location != uri) {
@@ -1205,12 +1236,12 @@ quick_lounge_load_uri_async (QuickLounge  *quick_lounge,
 		path_list_handle_free (quick_lounge->handle);
 	}
 
-	container = GTK_CONTAINER (quick_box);
-	for (children = GTK_BOX (quick_box)->children; children; ) {
-		GtkBoxChild *child = children->data;
-		gtk_container_remove (container, child->widget);
-		children = GTK_BOX (quick_box)->children;
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget *child_widget = scan->data;
+		gtk_widget_destroy (child_widget);
 	}
+	g_list_free (children);
 
 	quick_lounge->handle = path_list_async_new (quick_lounge->location,
 						    load_uri__step2,
@@ -1222,24 +1253,23 @@ void
 quick_lounge_load_uri_sync (QuickLounge  *quick_lounge, 
 			    const char   *uri)
 {
-	GtkContainer *container;
-	GList        *children;
+	GList        *children = NULL, *scan;
 	QuickBox     *quick_box = quick_lounge->quick_box;
 	GList        *file_list = NULL, *dir_list = NULL;
 	GList        *order_list = NULL;
-	GList        *files, *scan;
+	GList        *files;
 
 	if (quick_lounge->location != uri) {
 		g_free (quick_lounge->location);
 		quick_lounge->location = g_strdup (uri);
 	}
 
-	container = GTK_CONTAINER (quick_box);
-	for (children = GTK_BOX (quick_box)->children; children; ) {
-		GtkBoxChild *child = children->data;
-		gtk_container_remove (container, child->widget);
-		children = GTK_BOX (quick_box)->children;
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget *child_widget = scan->data;
+		gtk_widget_destroy (child_widget);
 	}
+	g_list_free (children);
 
 	if (! path_list_new (quick_lounge->location, &file_list, &dir_list))
 		return;
@@ -1424,14 +1454,6 @@ applet_button_press_cb (GtkWidget         *widget,
 }
 
 
-static void
-icon_theme_changed (GnomeIconTheme *icon_theme,
-		    QuickLounge    *quick_lounge)
-{
-	quick_lounge_load_uri (quick_lounge, quick_lounge->location);
-}
-
-
 static gboolean
 quick_lounge_applet_fill (PanelApplet *applet)
 {
@@ -1455,10 +1477,6 @@ quick_lounge_applet_fill (PanelApplet *applet)
 	quick_lounge->icon_theme = gnome_icon_theme_new ();
 	gnome_icon_theme_set_allow_svg (GNOME_ICON_THEME (quick_lounge->icon_theme), TRUE);
 	gnome_icon_theme_append_search_path (quick_lounge->icon_theme, PIXMAPSDIR);
-	g_signal_connect (G_OBJECT (quick_lounge->icon_theme),
-			  "changed",
-			  G_CALLBACK (icon_theme_changed),
-			  quick_lounge);
 
 	/**/
 
